<?php
/**
 * Implements the guts of the Fashion Traffic plugin.
 *
 * This class is designed to only be instantiated once. It's primary purpose is to hook WordPress actions and be a
 * container for the callables that those hooks need.
 *
 * The majority of the functionality for this plugin is implemented on Fashion Traffic's IFrame Server.
 *
 * @author Mike Schinkel <mike@newclarity.net>
 *
 * @since 1.0.0
 *
 */
class FashionTraffic_Plugin {

	/**
	 * @var FashionTraffic_Plugin Capture $this so others can use FashionTraffic_Plugin::self() with remove_action() if needed.
	 */
	private static $self;

	/**
	 * When object is instantiated verify this class is a singleton and hook 'init' action on creation of plugin.
	 */
	function __construct() {

		/**
		 * Ensure this class is only instantiated once.
		 */
		if ( isset( self::$self ) ) {
			echo __( 'The FashionTraffic_Plugin class is a singleton and can only be instantiated once, by the plugin itself.', 'fashiontraffic' );
			die(1);
		};

		/**
		 * Capture $this so  others can use FashionTraffic_Plugin::self() with remove_action() if needed.
		 */
		self::$self = $this;

		/**
		 * Hook 'init' so we can add other actions during initializations.
		 * Set priority to 9 so it will be run before default priorities thus making
		 * it easier for a theme or other plugin to remove the 'admin_head' action, if needed.
		 */
		add_action( 'init', array( $this, 'init' ), 9 );
	}

	/**
	 * Return the singleton instance of this class.
	 *
	 * To be use in case someone needs to remove one of the actions.
	 *
	 * @static
	 * @return FashionTraffic_Plugin
	 *
	 * @since 1.0.0
	 */
	static function self() {
		return self::$self;
	}

	/**
	 * Hook additional actions required by this plugin.
	 *
	 * @since 1.0.0
	 */
	function init() {

		/**
		 * Add AJAX listener for IFrame server to send a message requesting logout (i.e. clear auth.)
 		 */
		add_action( 'wp_ajax_fashiontraffic_clear_auth', array( $this, 'wp_ajax_fashiontraffic_clear_auth' ) );

		/**
		 * Add AJAX listener for IFrame server to send a message requesting storage of login credentials (i.e. set auth.)
 		 */
		add_action( 'wp_ajax_nopriv_fashiontraffic_set_auth', array( $this, 'wp_ajax_nopriv_fashiontraffic_set_auth' ) );

		/**
		 * Add action to allow plugin to define the Fashion Traffic metabox for posts and pages.
 		 */
		add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ) );

		/**
		 * Add action to output CSS for Fashion Traffic metabox for posts and pages.
		 * The could also be removed using this code in an 'init' hook priority 10 or later,
		 * or an 'admin_init' hook:
		 *
		 * 		if ( function_exists( 'remove_fashiontraffic_admin_css' ) )
		 *       remove_fashiontraffic_admin_css();
		 *
		 * Priority set to 0 so action will be run early thus enabling other CSS to easily override.
 		 */
		add_action( 'admin_head', array( $this, 'admin_head' ), 0 );

	}
	/**
	 * Add CSS to style the Fashion Traffic Metabox and contained IFrame.
	 *
	 * This action is run on priority 0 so it will be easy to override with other CSS.
	 * CSS is placed in the header rather an external CSS file because this is a trivial amount of CSS and
	 * more importantly external HTTP requests slow down page loads.
	 *
	 * @since 1.0.0
	 */
	function admin_head() {
		global $pagenow;

		/**
		 * Doing this here rather than hook 'admin_init' too!
		 * Only for the plugins page.
		 */
		if ( 'plugins.php' == $pagenow ) {
			/**
			 * Add a link naming the author.
			 */
			add_filter( 'plugin_row_meta', array( $this, 'plugin_row_meta' ), 10, 2 );
		}

		/**
		 * If we are not on an "Add New Post" (post-new.php) or "Edit Post" page (post.php)
		 * in the WordPress admin then return as there's no need to include this CSS.
		 */
		if ( ! preg_match( '#^(post.php|post-new.php)$#', $pagenow ) )
			return;

		$html = <<<HTML
<style type="text/css">
<!--
#fashiontrafficdiv .inside { margin: 0; padding: 0; }
#fashiontrafficdiv .handlediv, #fashiontrafficdiv .hndle { display: none }
#fashiontraffic-iframe { overflow: hidden; height:324px; width:278px; }
-->
</style>
HTML;
		echo $html;
	}

	/**
	 * Filter hook to add a new link under the plugin description.
	 *
	 * @param array $plugin_meta List of HTML links.
	 * @param string $plugin_file slug for the plugin
	 * @return array Filtered HTML links for the plugin page for this plugin.
	 *
	 * @since 1.0.0
	 */
	function plugin_row_meta( $plugin_meta, $plugin_file ) {
		/**
		 * Only add the link for this plugin.
		 */
		if ( false !== strpos( $plugin_file, '/fashiontraffic.php' ) ) {
			/**
			 * This will be displayed under the plugin description
			 * and after the text and links that look like this:
			 *
			 * 		Version X.X.X | By Fashion Traffic LTD | Visit plugin site
			 *
			 */
			$link_text = __( "Visit developer's site", 'fashiontraffic' );
			/**
			 * This points to the about.me page for the President of NewClarity LLC
			 * instead of the website for NewClarity LLC because the latter's website
			 * leaves a lot to be desired at the time of publishing.
			 */
			$plugin_meta[] = "<a href=\"http://about.me/mikeschinkel\" target=\"_blank\">{$link_text}</a>";
		}
		return $plugin_meta;
	}

	/**
	 * Add the Fashion Traffic metabox to "Add New Post" and "Edit Post" pages in the WordPress admin.
	 *
	 * @since 1.0.0
	 */
	function add_meta_boxes( $post_type ) {

		/**
		 * Grab list of valid post types for the Fashion Traffic plugin,
		 * ensure the post type presented is one of the valid ones, bail if not.
		 */
		$post_types = implode( '|', $this->get_post_types() );
		if ( ! preg_match( "#^({$post_types})$#", $post_type ) )
			return;

		/**
		 * Add the meta box.
		 * add_meta_box( $metabox_id, $title, $callback, $post_type, $context, $priority, $callback_args );
		 */
		add_meta_box(
			'fashiontrafficdiv',
			__( 'Find and Post your Fashion', 'fashiontraffic' ),
			array( $this, 'metabox' ),
			$post_type,
			'side',	// Metabox gets displayed on right 'side', not 'normal' (center high) or 'advanced' (center low)
			'high'	// Metabox gets 'high' on the right side, not 'core', 'default' of 'low'
		);
	}

	/**
	 * Gets a list of post type(s) to have Fashion Traffic metaboxes.
	 *
	 * Use this method in 'admin_init' hook.
	 *
	 * @return array One of more post types strings.
	 *
	 * @since 1.0.0
	 */
	function get_post_types() {
		/**
		 * Get the array of available post types for the current WordPress install.
		 */
		global $wp_post_types;

		/**
		 * Make an array to hold the return values.
		 */
		$post_types = array();

		/**
		 * Loop through the post types and grab the object that defines each post type
		 */
		foreach( $wp_post_types as $post_type_object ) {

			/**
			 * If there IS a 'fashiontraffic' property for the post type object and it is true
			 * make this a valid post type for Fashion Traffic.
			 */
			if ( isset( $post_type_object->fashiontraffic ) && $post_type_object->fashiontraffic ) {
				$post_types[$post_type] = $post_type;
			}

		}
		/**
		 * If we've not already set 'post' and it doesn't have a fashiontraffic property,
		 * add 'post' to the list of post types that should get Fashion Traffic metaboxes.
		 */
		if ( ! isset( $post_types['post'] ) && ! isset( $wp_post_types['post']->fashiontraffic ) ) {
			$post_types[] = 'post';
		}

		return $post_types;
	}

	/**
	 * Display the Fashion Traffic UI in the post editor within the WordPress admin.
	 *
	 * @param object $post stdClass post object, same as "$GLOBALS['post']", contains info about the post being edited.
	 * @param array $metabox Represents the metabox, likely contains elements 'id', 'title', 'callback' and 'args'.
	 *
	 * @since 1.0.0
	 */
	function metabox( $post, $metabox ) {

		/**
		 * Grab value of FASHIONTRAFFIC_IFRAME_SERVER_URL into local var for use in HEREDOC
		 * @var string Base URL for the IFrame server that powers this plugin.
		 *
		 */
		$iframe_base_url = FASHIONTRAFFIC_IFRAME_SERVER_URL;

		/**
		 * Create and store a nonce so that nopriv ajax callbacks can verify before updating auth.
		 */
		$nonce = wp_create_nonce('fashiontraffic');
		update_option( 'fashiontraffic_nonce', $nonce );

		/**
		 * Generate and encode URL for messenger.php included in this plugin for
		 * passing as a URL parameter for the URL that we will IFrame below.
		 */
		$messenger_url = urlencode( plugins_url( '/messenger.php', dirname( __FILE__ ) ) );

		/**
		 * Generate and encode URL for WordPress' admin-ajax.php for passing
		 * as a URL parameter for the URL that we will IFrame below.
		 */
		$ajax_url = urlencode( admin_url( 'admin-ajax.php' ) );

		/**
		 * Load auth (includes
		 */
		$auth = urlencode( get_option( 'fashiontraffic_auth' ) );
		if ( ! empty( $auth ) )
			$auth= "&auth={$auth}";

		/**
		 * Compose URL for IFrame.
		 *
		 * This IFrame will load a .PHP file that contains two sub-iframes; one on the IFrame server and
		 * one to IFrame back into the messenger.php included with this plugin. NewClarity created this
		 * technique to get around cross-domain message passing.
		 */
		$iframe_src = "http://{$iframe_base_url}/?messengerurl={$messenger_url}&ajaxurl={$ajax_url}&nonce={$nonce}{$auth}";

		/**
		 * Get the Javascript code to display before the IFrame. This code receives messages from and passes messages back
		 * to the iFrame server using the parent.parent.sendMessage() technique to get past cross-domain limitations of
		 * Javascript.
		 */
			$messenger_js = $this->get_messenger_js();

		$html = <<<HTML
<script type="text/javascript">
{$messenger_js}
</script>
<iframe id="fashiontraffic-iframe" src="{$iframe_src}" scrolling="no"></iframe>
HTML;
		echo $html;
	}

	/**
	 * Get the Javascript code to display before the IFrame. This code receives messages from and passes messages back
	 * to the iFrame server using the parent.parent.sendMessage() technique to get past cross-domain limitations of
	 * Javascript.
	 *
	 * Javascript is placed inline in the HTML because it is only needed if the metabox is placed and
	 * more importantly external HTTP requests slow down page loads, although with HTTP caching it
	 * could be insignificant. TODO: Consider externalizing Javascript.
	 *
	 * The code is not commented to keep the Javascript small. TODO: Minify Javascript.
	 *
	 * @return string The Javascript.
	 */
	function get_messenger_js() {
		$js = <<<JS
var pluginMessenger;
jQuery(document).ready(function($){
	pluginMessenger = {
		container: $('#fashiontrafficdiv'),
		sendMessage: function(message){
			var nameValue = (message+':').split(':');
			nameValue[1] = decodeURIComponent(nameValue[1]);
			switch (nameValue[0]) {
				case 'logout':
					this.hidePopup();
					this.clearAuth();
					break;
				case 'add-link':
					this.addLink(nameValue[1]);
					break;
				case 'add-content':
					this.addContent(nameValue[1]);
					break;
				case 'show-popup':
					this.showPopup(nameValue[1]);
					break;
				case 'hide-popup':
					this.hidePopup();
					break;
				default:
					break;
			}
		},
		addLink: function (content) {
			this.insertTinyMceContent(' '+content+' ');
		},
		addContent: function(data) {
			data = this.parseParams( data );
			var outerThis = this;
			$.get(decodeURIComponent(data.url)+'&auth='+data.auth, function(html) {
				outerThis.insertTinyMceContent(html);
				}, 'html'
			);
		},
		showPopup: function(data) {
			$("#fashiontraffic-popup").remove();
			var iFrame = $("#fashiontraffic-iframe").offset();
			data = this.parseParams( data );
			var property;
			for( property in {height:0,width:0,top:0,left:0} ) {
				data[property] = parseInt(data[property]);
			}
			var top = data.top + iFrame.top;
			var left = data.left + iFrame.left - data.width;
			var url = decodeURIComponent(data.url)+'&auth='+data.auth+'&messengerurl='+data.messengerUrl;
			var popup = $('<iframe>')
				.attr('id','fashiontraffic-popup')
				.attr('src',url)
				.css({
					position: "absolute",
					top: top+'px',
					left: left+'px',
					height: data.height+'px',
					width: data.width+'px',
					padding: '15px',
					border: '1px solid #d8d8d8',
					backgroundColor: '#f8f8f8'
			});
			$(popup).hover(function () {
				$(this).addClass('hovered');
			}, function () {
				$(this).removeClass('hovered');
			});
			$(document.body).append(popup);
		},
		hidePopup: function( data ) {
			$("#fashiontraffic-popup").remove();
		},
		clearAuth: function() {
			var data = {
				action: 'fashion_trffic_clear_auth',
				async: false
			};
			$.post(ajaxurl,data,function(response) {
				if ('success'!=response) {
					alert( "Hmm. No Success. Hmm." );
					console.log( "clearAuth failed." );
				} else {
					console.log( "clearAuth succeeded." );
				}
			});
		},
		parseParams: function( params ) {
			var result = {};
			var parts;
			params = params.split("&");
			for(var i=0;i<params.length;i++) {
				parts = params[i].split("=");
				result[parts[0]] = parts[1]
			}
			return result;
		},
		insertTinyMceContent: function (content) {
			if ($('#wp-content-wrap').hasClass('html-active')) {
				$('#content').val($('#content').val()+content);
			} else {
				tinyMCE.execCommand('mceInsertContent', false, content)
			}
		}

	};
});
jQuery(document).ready(function($){
	$('#closer').live('click',function() {
		messenger.postMessage('hide-popup');
	});
});

JS;
		return $js;
	}

	/**
	 * Store authorization passed from IFrame server to send back during subsequent calls to the IFrame server.
	 *
	 * $_POST is expected to contain 'auth' and 'nonce' elements.
	 *
	 * @since 1.0.0
	 */
	function wp_ajax_nopriv_fashiontraffic_set_auth() {

		/**
		 * Pull an 'auth' value out of $_POST and sanitize it.
		 * This 'auth' value is generated by the IFrame server
		 * upon username and password authentication and is
		 * opaque to this plugin.
		 */
		$auth = filter_input( INPUT_POST, 'auth', FILTER_SANITIZE_STRING );

		/**
		 * Pull an 'nonce' value out of $_POST and sanitize it.
		 * This nonce (Number used ONCE) value comes from the nonce
		 * generated in the $this->metabox() method that was stored
		 * in wp_options in the 'fashiontraffic_nonce' key.
		 */
		$nonce = filter_input( INPUT_POST, 'nonce', FILTER_SANITIZE_STRING );

		/**
		 * If the nonce is the same one we created in a most recent page load
		 * we can be pretty sure it's not a hacker so we can store the auth
		 * passed to us from the IFrame server.
		 */
		if ( get_option( 'fashiontraffic_nonce' ) == $nonce )
			update_option( 'fashiontraffic_auth', $auth );

		/**
		 * The AJAX function always 'succeeds.' Let the caller know this was a 'success.'
		 */
		echo 'success';

		/**
		 * Halt further processing to keep WordPress from echoing out a '1' before it quits.
		 */
		exit;
	}
	/**
	 * Remove previously stored authorization which will force the login screen to be displayed
	 * in the Fashion Traffic metabox next time.
	 *
	 * Nothing is expected in $_POST.
	 *
	 * @since 1.0.0
	 */
	function wp_ajax_fashiontraffic_clear_auth() {

		/**
		 * If this is called, just clear the auth, no need to check a nonce.
		 * No real harm can come from doing this, only potential annoyance
		 * although the likelyhood of this AJAX request being hacked is
		 * slim to none.
		 */
		delete_option( 'fashiontraffic_auth' );

		/**
		 * The AJAX function always 'succeeds.' Let the caller know this was a 'success.'
		 */
		echo 'success';

		/**
		 * Halt further processing to keep WordPress from echoing out a '1' before it quits.
		 */
		exit;
	}
}

/**
 * Instantiate this class to ensure the action hooks are hooked.
 * This instantiation can only be done once (see it's __construct() to understand why.)
 */
new FashionTraffic_Plugin;
